#!/usr/bin/perl

use strict;
use warnings;

use Test::More;
eval "use Test::Pod 1.00";
plan skip_all => "Test::Pod 1.00 required for testing POD" if $@;

plan tests => 2;
use_ok("jsFind");

pod_file_ok($INC{"jsFind.pm"});

